<?php
/**
*   Software-update check class
*   @author Christian Weiske <cweiske@cweiske.de>
*
*   Can be used to determine if a new software version
*   is available
*
*   The software information file is required to be in 
*   XSA (XML Software Autoupdate) format
*
*   Requires the XMLFile class (xml.inc) by Christian Weiske
*
*   Usage example:
*   if( $uc->isUpdateAvailable( '/data/html/cweiske/www.cweiske.de/phpgtk_manualbrowser.xsa', 'toc-en', '0.1', null)) {
*       var_dump( $uc->getUpdateInformation( '/data/html/cweiske/www.cweiske.de/phpgtk_manualbrowser.xsa', 'toc-en'));
*   }
*
*   @todo
*   - cache information arrays and do the clearCache with them
*/


/**
*   Software-update check class
*   @author Christian Weiske <cweiske@cweiske.de>
*/
class UpdateChecker
{
    var $strLastXsaUrl      = null;
    var $strLastProductId   = null;
    
    /**
    *   The XML objects of the software update files
    *   are cached here
    *   @var array
    */
    var $arXmlCache         = array();
    
    var $strError           = null;
    
    
    /**
    *   constructor
    */
    function UpdateChecker()
    {
        if( !class_exists( 'XMLFile')) {
            trigger_error( 'XMLFile class not available');
            return false;
        }
    }//function UpdateChecker()
    
    
    
    /**
    *   checks if an update is available
    *   The version id OR the release date has to be given (and the other NULL)
    *   If none of both is given, FALSE is returned
    *   If both are given, only the version is used
    *   If the update information is not available, FALSE is returned
    *
    *   To get the changes and the URLs of the update,
    *   please use the getUpdateInformation function
    *
    *   @param  string  The URL of the update information file
    *   @param  string  The product id as in '<product id="gettext">'
    *   @param  string  The version of the software (x.y.z) (optional)
    *   @param  string  The current date of the software (optional) (format: timestamp OR US date format (Y-m-d))
    *
    *   @return boolean If a new version is available
    */
    function isUpdateAvailable( $strXsaUrl, $strProductId, $strVersion = null, $strDate = null)
    {
        if( $strVersion == null && $strDate == null) {
            trigger_error( 'UpdateChecker::isUpdateAvailable: no version and no date given');
            return false;
        }
        
        $arInformation  = $this->getUpdateInformation( $strXsaUrl, $strProductId);
        if( $arInformation === false) {
            return false;
        }
        
        $bAvailable = false;
        if( $strVersion != null) 
        {   //compare versions
            $strNewVersion  = $arInformation['version'];
            $bAvailable     = $this->isVersionNewer( $strNewVersion, $strVersion);
        }
        else
        {   //compare dates
            $strNewDate     = $arInformation['last-release'];
            if( strpos( $strDate, '-') !== false) {
                //US-Date Y-m-d
                $strCurrentDate    = str_replace( '-', '', $strDate);
            } else if( strpos( $strDate, '.') !== false) {
                //European date d.m.Y
                $arDate = explode( '.', $strDate);
                $strCurrentDate    = $arDate[2] . $arDate[1] . $arDate[0]; 
            } else {
                //timestamp
                $strCurrentDate    = date( 'Ymd', $strDate);
            }
            //current and new date are integers now - just compare them
            $bAvailable = $strNewDate > $strCurrentDate;
        }//compare dates
        
        return $bAvailable;
    }//function isUpdateAvailable( $strXsaUrl, $strSoftwareId, $nVersion = null, $strDate = null)
    
    
    
    /**
    *   returns information on the current software version
    *   like
    *   - Name
    *   - Changes
    *   - Information URL
    *   - ...
    *
    *   You need to pass the URL of the file and the product id
    *   to be able to handle different update files with one
    *   instance of this class. If you don't want that, use
    *   the getLastUpdateInformation function
    *
    *   @param  string  The URL of the update information file
    *   @param  string  The product id as in '<product id="gettext">'
    *
    *   @return array   An array full of information about the update | boolean false if there is no update information available
    */
    function getUpdateInformation( $strXsaUrl, $strProductId)
    {
        $this->strError         = null;//no error until now :)
        $this->strLastXsaUrl    = $strXsaUrl;
        $this->strLastProductId = $strProductId;
        
        if( !isset( $this->arXmlCache[$strXsaUrl]))
        {
            //no xml available - load it
            $strContent     = @file_get_contents( $strXsaUrl);
            if( $strContent == '') {
                //no content
                $this->strError = 'Update information file' . "\n" . $strXsaUrl . "\n" . 'is not available';
                return false;
            }
            $xmlfile             = new XMLFile();
            $xmlfile->strXMLtext = $strContent;
            if( !$xmlfile->Load()) {
                //error loading xml
                $this->strError = 'Error loading xml';
                return false;
            }
            if( !isset( $xmlfile->XMLObject->xsa)) {
                //no xsa element
                $this->strError = 'XML doesn\'t contain an xsa root element';
                return false;
            }
            //make product array
            XMLFile::MakeArray( $xmlfile->XMLObject->xsa, 'product');
            $this->arXmlCache[$strXsaUrl]   = $xmlfile->XMLObject;
        }//need to load the xml
        
        $xobj   = &$this->arXmlCache[$strXsaUrl];
        
        if( !isset( $xobj->xsa->product) || !is_array( $xobj->xsa->product) || count( $xobj->xsa->product) < 1) {
            //no product information available
            $this->strError = 'No product information available';
            return false;
        }
        
        //find the right product
        $bFound = false;
        foreach( $xobj->xsa->product as $objProduct)
        {
            if( $objProduct->Attributes->id == $strProductId) {
                $bFound = true;
                break;
            }
        }//foreach product
        
        if( $bFound === false) {
            //product not found
            $this->strError = 'Product not found';
            return false;
        }
        
        //make information array
        $arInformation  = array();
        foreach( $objProduct as $strId => $objValue)
        {
            if( $strId != 'Attributes') {
                $arInformation[$strId]  = $objValue->Value;
            }
            if( $strId == 'download-url' && isset( $objValue->Attributes->size)) {
                $arInformation['download-size'] = $objValue->Attributes->size;
            }
        }
        
        return $arInformation;
    }//function getUpdateInformation( $strXsaUrl, $strProductId)
    
    
    
    /**
    *   returns the update information of the last checked software
    *   by isUpdateAvailable
    *   @see getUpdateInformation
    *   @return array   An array full of information about the update | boolean false if there is no update information available
    */
    function getLastUpdateInformation()
    {
        return $this->getUpdateInformation( $this->strLastXsaUrl, $this->strLastProductId);
    }//function getLastUpdateInformation()
    
    
    
    /**
    *   compares the two version information and tells you
    *   if the first is newer than the latter
    *
    *   @param  string  The new version
    *   @param  string  The current version
    *
    *   @return boolean If the first version is newer than the latter
    */
    function isVersionNewer( $strNewVersion, $strCurrentVersion)
    {
        $strNewVersion      = str_replace( ',', '.', $strNewVersion);
        $strCurrentVersion  = str_replace( ',', '.', $strCurrentVersion);
        
        return version_compare( $strNewVersion, $strCurrentVersion, '>');
    }//function isVersionNewer( $strNewVersion, $strCurrentVersion)
    
    
    
    /**
    *   The software update files are cached after they have been downloaded
    *   To clear this cache (force a reload of the file) call this function
    */
    function clearCache()
    {
        $this->arXmlCache   = array();
    }//function clearCache()
    
    
    
    /**
    *   returns the last error message
    *   @return string  The error message (NULL if no error)
    */
    function getLastError()
    {
        return $this->strError;
    }//function getLastError()
    
    
    
    /**
    *   returns if there has been an error
    *   @return boolean If there has been an error
    */
    function hadError()
    {
        return $this->strError !== null;
    }
}//class UpdateChecker
?>